/*
Copyright 2007, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2007
*/

/**
 * @file			ScatterWeb.Accelerator.c
 * @ingroup	libAccelerator
 * @brief			library providing specific functions to use the accelerator
 * 
 * @author		Christian Rotar	<rotar@inf.fu-berlin.de>
 * @date			Oct 2007
 * @since		1.1
 * @version	$Rev: 571 $
 * 
 */

#include "ScatterWeb.Accelerator.internals.h"
#include "ScatterWeb.Accelerator.h"


/******************************************************************************
 * @name	Initialization and configuration
 * @{
 */



typedef struct {
  uint16_t x;
  uint16_t y;
  uint16_t z;
} TD_t;

typedef struct {
	unsigned char negEncX; 		//Kodierung fr Mausrichtung auf X - Achse (links/rechts)
	unsigned char negEncY;		//Kodierung fr Mausrichtung auf Y - Achse (hoch/runter)
	unsigned char stepEncX[3];	//Kodierung der 3 Stufen fr X-Achse
	unsigned char stepEncY[3];	//Kodierung der 3 Stufen fr Y-Achse
	unsigned char clickEnc;		//Kodierung ob Klick ist
	uint16_t stepRange[3];		//Bereichseingrenzung der X und Y-Achsen fr die 3 Stufen
}mouse_t;

TD_t* Data_get3D(TD_t* value);

TD_t NullBase(){
	TD_t tmp;
	tmp.x = 0;
	tmp.y = 0;
	tmp.z = 0;

	return tmp;
}



TD_t* Data_get3D(TD_t* value) { 
  value->x = ADC12MEM0;
  value->y = ADC12MEM3;
  value->z = ADC12MEM6;
  return value;
}


/*************************************************************************************
/Declarations */

TD_t base;
TD_t sensitySoft;
mouse_t mouse;


uint16_t cI; //CalibrationInterval
uint16_t wI; //Watch LoopInterval

void SetDefMouseEncValues();
void SetMouseStepRanges(uint16_t r1, uint16_t r2);
void sendData(TD_t r);
bool isOutOfRange(TD_t coord, int16_t range_x, int16_t range_y, int16_t range_z);
bool cmpTD(TD_t s1, TD_t s2);

void SimpleCalibration();
void Calibration_Loop(void* p);
void StartWatch_Loop(void* p);

void libAccelerator_Init() {
}


//***************************************************************************************


void LED_ON(){
	P5OUT &= ~0x80;
}

void LED_OFF(){
	P5OUT |=  0x80;
}

// Primitive Methode, die die aktuellen Koordinaten ausgibt
void GetAcceleratorValue(void* c){
	TD_t r;
	if (c==0) return;
	printf("P = %u ", (uint16_t) c);

	//Accelerator Wert in r schreibne
	Data_get3D( (TD_t*) &r);
	c = c - 1;
	
	printf("(%u, %u, %u) \r\n", r.x, r.y, r.z);
	Timers_add(1024, GetAcceleratorValue, c);
}

/*Vergleicht zwei TD_t Sturkturen auf gleichheit
 */
bool cmpTD(TD_t s1, TD_t s2){
	return s1.x == s2.x && s1.y == s2.y && s1.z == s2.z;
}




/* Startet eine Routine, die die Koordinaten auf Bewegung berprft.
 * Die Routine konfiguriert sich selbstndig und zeigt eine Meldung an,
 * falls sich der Wert einer Korrdinate seit der Konfiguration um +- sensitySoft verndert hat.
 * Umso hher sensitySoft, desto weniger schlgt die Routine alarm!
 * CalibrationInterval ist die Hufigkeit (ca in Millisekunden),
 * wie oft sich das Modul neu Kalibriert. Bei CalibrationInterval = 0
 * wird nur ein mal Kalibriert.
 */
void StartWatch(int16_t sensitySoft_x,
			    int16_t sensitySoft_y,
			    int16_t sensitySoft_z,
			    uint16_t loopInterval,
			    uint16_t mouseStepRange1,
			    uint16_t mouseStepRange2){
	
	if (cmpTD(base, NullBase())) {
		//Falls Kalibration vorher nicht stattgefunden, Fehler ausgeben
		printf("//Vor dem Ausfhren von StartWatch muss Calibration ausgefhrt werden!");
		return;
	}
	
	SetMouseStepRanges(mouseStepRange1, mouseStepRange2);

	sensitySoft.x = sensitySoft_x;
	sensitySoft.y = sensitySoft_y;
	sensitySoft.z = sensitySoft_z;
	wI = loopInterval;
	stopWatch=False;
	
	StartWatch_Loop(NULL);
}
void StartWatch_Loop(void* p){
	
	//Accelerator Wert in r schreiben
	TD_t r;
	Data_get3D( (TD_t*) &r);
	
	
	if (isOutOfRange(r, sensitySoft.x, sensitySoft.y, sensitySoft.z)){
		//Daten ausgeben
		sendData(r);
		//printf("\r\nCoord: %u,%u,%u\r\n", r.x, r.y, r.z);
	}
	
	
	if(stopWatch == True) {
		//printf("//Watch terminated \r\n");
		LED_OFF();
		wI=0;
		return;
	}
		
	Timers_add(wI, StartWatch_Loop, NULL);
}




/*
 * Fhrt die Calibration-Methode im angegebenen Interval aus.
 * Falls runOnce = 0 ist, wird sich die die Methode im angegebenen Intervall
 * selbst aufrufen. Falls runOnce>0 wird die Methode nur ein Mal ausgefhrt
 */
void Calibration(uint16_t runOnce, uint16_t interval){
	
	SimpleCalibration();
	
	if (runOnce == 0){
		//Loop starten
		stopCal=False;
		cI=interval;
		Calibration_Loop(NULL);
	}	
	
}
void Calibration_Loop(void* p){
	LED_OFF();
	if(stopCal == True) {
		//printf("//CycleCalibration terminated \r\n");
		//base = NullBase();
		return;
	}
		
	SimpleCalibration();
		
	//Fortlaufende Kalibrierung nur dann, wenn der Parameter > 0 ist
	Timers_add(cI, Calibration_Loop, NULL);
}

//Setzt den base Struct auf die aktuell gelesenen Koordinaten
void SimpleCalibration(){
	//Accelerator Wert in r schreibne
	TD_t r;
	Data_get3D( (TD_t*) &r);

	base = r;
}

/*
 * Stellt die Kodierungen fr die mouse-Variable richtig ein
 * */
void SetDefMouseEncValues(){
	mouse.negEncX = 0x80; 		//1000 0000
	mouse.stepEncX[0] = 0x20;	// 000 0000
	mouse.stepEncX[1] = 0x40;	// 010 0000
	mouse.stepEncX[2] = 0x60;	// 100 0000
	mouse.negEncY = 0x08; 		//	   1000
	mouse.stepEncY[0] = 0x02;	// 		000
	mouse.stepEncY[1] = 0x04;	// 		010
	mouse.stepEncY[2] = 0x06;	// 		100
}

//Stellt fr die mouse-Variable die StepRanges ein
void SetMouseStepRanges(uint16_t r1, uint16_t r2){
	mouse.stepRange[0]=0;
	mouse.stepRange[1]=r1;
	mouse.stepRange[2]=r2;
}




//Gibt aus, ob alle coordinaten innerhalb von base +- range liegen
bool isOutOfRange(TD_t coord, 
				  int16_t range_x,
				  int16_t range_y,
				  int16_t range_z){
	
	return (range_x > -1 && (coord.x < base.x - range_x || coord.x > base.x + range_x)) ||
		   (range_y > -1 && (coord.y < base.y - range_y || coord.y > base.y + range_y)) ||
	       (range_z > -1 && (coord.z < base.z - range_z || coord.z > base.z + range_z));
}

void sendMData(){
	sendData(NullBase());
}

void sendData(TD_t r){
	
	unsigned char c = 0x00;
	unsigned char xfound = 0x00;
	unsigned char yfound = 0x00;
	uint16_t steps =3;
	int16_t i;
	
	//printf("Steps: %u \r\n", steps);
	//X-Achse
	if (r.x != 0){
		for (i=steps-1;i>=0;i--){
			if (r.x <= base.x - sensitySoft.x - mouse.stepRange[i]){
				c |= mouse.negEncX;
				c |= mouse.stepEncX[i];
				xfound=0x01;
				//printf("rechts %u %u \r\n", i, mouse.stepRange[i]); //1Byte = 8Bit senden
				break;
			}
			//printf("R[%u] %u <= %u \r\n", i, r.x, base.x - sensitySoft.x - mouse.stepRange[i]); //1Byte = 8Bit senden
		}
		if (xfound==0x00){
			for (i=steps-1;i>=0;i--){
				if (r.x >= base.x + sensitySoft.x + mouse.stepRange[i]){
					c &= ~mouse.negEncX;
					c |= mouse.stepEncX[i];
					//printf("links %u %u \r\n", i, mouse.stepRange[i]); //1Byte = 8Bit senden
					break;
				}
			}
			//printf("L[%u] %u >= %u \r\n", i, r.x, base.x + sensitySoft.x + mouse.stepRange[i]); //1Byte = 8Bit senden
		}
	}

	//Y-Achse
	if (r.y != 0){
		for (i=steps-1;i>=0;i--){
			if (r.y <= base.y - sensitySoft.y - mouse.stepRange[i]){
				c |= mouse.negEncY;
				c |= mouse.stepEncY[i];
				yfound=1;
				break;
			}
		}
		if (yfound==0){
			for (i=steps-1;i>=0;i--){
				if (r.y >= base.y + sensitySoft.y + mouse.stepRange[i]){
					c &= ~mouse.negEncY;
					c |= mouse.stepEncY[i];
					break;
				}
			}
		}
	}
	
	if (button1){
		c |= button1;
		button1=0x00;
	}
	if (button2){
		c |= button2;
		button2=0x00;
	}

	printf("%c", c); //1Byte = 8Bit senden
	//printf("%c", 0x81);

}

void showInfos(){
	printf("   \r\n"); //Diese Zeile ist ntig, damit korrekt geparst werden kann
	printf("Calibrated: %u \r\n", !cmpTD(base, NullBase()));
	printf("Started: %u \r\n", wI>0);
	printf("WatchRate: %u \r\n", wI);
	
	printf("Base: \r\n");
	printf("x: %u \r\ny: %u \r\nz: %u \r\n", base.x, base.y, base.z);
	
	printf("SensitySoft: \r\n");
	printf("x: %u \r\ny: %u \r\nz: %u \r\n", sensitySoft.x, sensitySoft.y, sensitySoft.z);
	
	printf("MouseStepRange: \r\n");
	uint16_t i;
	for(i=0;i<3;i++){
		printf("[%u]: %u \r\n", i, mouse.stepRange[i]);
	}
	
	printf("MouseStepEncodings: \r\n");
	for(i=0;i<3;i++){
		printf("x[%u]: %u \r\n", i, mouse.stepEncX[i]);
	}	
	for(i=0;i<3;i++){
		printf("y[%u]: %u \r\n", i, mouse.stepEncY[i]);
	}
	
	printf("MouseNegEncX: %u \r\n", mouse.negEncX);
	printf("MouseNegEncY: %u \r\n", mouse.negEncY);
}



/** @} */

