#include "fat.internals.h"
// #include <stdio.h> // add for printf support

uint8_t get_lowest_bit_index(register int i) {
	register uint8_t bit = 0;
	
	do {
		if( i & 0x01 )
			return bit;
		i >>= 1;
		bit++;
	} while(i != 0);

	return 0;
}

void libFAT_Init() {
	
};

enum fat_init_result Fat_Init(fat_state_t* pFS) {
	uint8_t		ret;
	uint32_t		address;
	uint16_t		BytsPerSec;
	fat_sa_t	BpbSA;			///< first sector of VBR
	
	struct fat_partition	part;
	struct fat_partition*	pPart = &part;
	struct fat				fatTbl;
	struct fat*				pFAT = &fatTbl;
	struct fat_bpb*			pBPB = (struct fat_bpb*)&fatTbl;

	memset(pFS, 0, sizeof(fat_state_t));

	if( sizeof(struct fat_bpb) != 25 ) {
		// if this fails, structure packing does not work correctly
		return FAT_STRUCT_PACKING;
	}

	// test for BIOS signature
	#if !SD_CACHE
		Sd_set_blocklength(1); // 2
	#endif
	ret = Sd_read(pFAT, 510, 2);
	// BIOS signature (found in MBR and VBR)
	if( ((uint16_t*)pFAT)[0] != FAT_BIOS_SIGNATURE )
		return FAT_ERROR_SPEC;
	
	// read vbr and partition table from first sector and
	// decide which one exists
	#if !SD_CACHE
		Sd_set_blocklength(7); // 128
	#endif
	// read VBR
	ret = Sd_read( pFAT, FAT_BPB_OFFSET, sizeof(struct fat) );		
	if( ret == 0 ) return FAT_ERROR_READ;
	// read 1. partition table entry
	ret = Sd_read( pPart, 0x01BE, sizeof(struct fat_partition) );	
	if( ret == 0 ) return FAT_ERROR_READ;

	//
	// decide if MBR or VBR
	//
	ret = 0;
	// MBR with FAT16 partition
	if( (pPart->type == 4) || (pPart->type == 6) )
		ret |= 0x01;
	// sanity check for valid FAT BPB
	if( (pFAT->bpb.SecPerTrk > 0) && (pFAT->bpb.BytsPerSec > 0) && (pFAT->bpb.SecPerClus <= 64) )
		ret |= 0x04;
	
	//
	// handle MBR
	//
	if( ret == 0x01 ) {	
		// is likely to be MBR
		BpbSA = pPart->distance_to_first;
		address = FAT_BPB_OFFSET + (BpbSA << 9);

		// read VBR
		ret = Sd_read( pFAT, address, sizeof(struct fat) );
		if( ret == 0 ) return FAT_ERROR_READ;
	} else {
		BpbSA = 0;
	}

	//
	// handle VBR
	//
	
	// copy BPB values (note MSB430 has issues with word which are not word-aligned...)
	uint16_t u16tmp;
		
	pFS->RsvdSecCnt = ((uint8_t*)&pFAT->bpb)[3];
	pFS->RsvdSecCnt+= ((uint8_t*)&pFAT->bpb)[4] << 8;
	pFS->NumFATs = pBPB->NumFATs;			// usually 2
	pFS->RootEntCnt = pFAT->bpb.RootEntCnt;
	
	if( pFAT->bpb.TotSec16 != 0 )
		pFS->TotSec = pFAT->bpb.TotSec16;
	else {
		u16tmp = ((uint8_t*)&pFAT->bpb)[23];
		u16tmp+= ((uint8_t*)&pFAT->bpb)[24] << 8;
		pFS->TotSec = u16tmp;
	}
	
	u16tmp = ((uint8_t*)&pFAT->bpb)[11];
	u16tmp+= ((uint8_t*)&pFAT->bpb)[12] << 8;
	if( u16tmp != 0 )
		pFS->FATSz = u16tmp;
	else {
		#if FAT32_SUPPORT
			memcpy(&pFS->FATSz, &pFAT->vbr.fat32.FATSz32, 4);
		#else
			/// TODO: FAT32
			return FAT_ERROR_TYPE;
		#endif
	}

	/*
	printf("BytsPerSec %.4x\n"
		   "SecPerClus %.2x\n"
		   "RsvdSecCnt %.4x\n"
		   "NumFats	   %.2x\n"
		   "RootEntCnt %.4x\n",
		pFAT->bpb.BytsPerSec,
		pFAT->bpb.SecPerClus,
		pFS->RsvdSecCnt,
		pFAT->bpb.NumFATs,
		pFAT->bpb.RootEntCnt
	);
	*/

	// precalculation
	BytsPerSec = pFAT->bpb.BytsPerSec;	
	pFS->BytsPerSec_bit = get_lowest_bit_index( BytsPerSec );
	pFS->SecPerClus_bit = get_lowest_bit_index( pFAT->bpb.SecPerClus );
	pFS->BytsPerClus_bit = pFS->BytsPerSec_bit + pFS->SecPerClus_bit;
	pFS->RootDirSz = ((pFS->RootEntCnt << 5) + (BytsPerSec - 1)) >> pFS->BytsPerSec_bit;
	pFS->FatSA = BpbSA + pFS->RsvdSecCnt;
	pFS->FatA = pFS->FatSA << pFS->BytsPerSec_bit;
	pFS->FirstClusterSA = pFS->FatSA + (pFS->NumFATs * pFS->FATSz) + pFS->RootDirSz;
	
	#if FAT_WRITE
		pFS->LastFreeCA = 0;
	#endif
	
	pFS->FATType = _Fat_GetType(pFS, pFAT);
	#if !FAT12_SUPPORT
		if( pFS->FATType & FAT12 )
			return FAT_ERROR_TYPE;
	#endif
	#if !FAT32_SUPPORT
		if( pFS->FATType & FAT32 )
			return FAT_ERROR_TYPE;
	#endif

	// first sector of root directory
	#if FAT32_SUPPORT
		if( pFS->FATTYPE & FAT32 )
			pFS->RootDirCA = pFAT->vbr.fat32.RootClus;
		else
			pFS->RootDirCA = 1;
	#else
		pFS->RootDirCA = 1;
	#endif
	
	return FAT_ERROR_NONE;
}

void _Fat_getClusterFAT(const fat_state_t* pFS, fat_ca_t* pCA, uint32_t* pAddress) {
	//
	// calculate FAT entry for cluster
	//

	*pAddress = *pCA;
	#if FAT12_SUPPORT
		if( pFS->FATType == FAT12 )
			*pAddress += *pCA >> 1;
		else
			*pAddress <<= pFS->FATType;
	#else
		*pAddress <<= pFS->FATType;
	#endif
	*pAddress += pFS->FatA;	
}

void _Fat_getClusterPos(const fat_state_t* pFS, const fat_ca_t* pCA, uint32_t* pAddress) {
	*pAddress = pFS->FirstClusterSA + *pCA - FAT_CLUSTER_OFFSET;
	*pAddress <<= pFS->BytsPerClus_bit;
}

bool _fat_getNextCA(const fat_state_t* pFS, fat_ca_t cluster, fat_ca_t* pNextCluster) {
	uint32_t	address;	
	uint8_t 	bl;
	bool			ret;		// mult-purpose response buffer

	_Fat_getClusterFAT( pFS, &cluster, &address );
	//
	// read FAT
	//
	bl = FAT_ENTRY_WIDTH(pFS);
	ret = Sd_read( pNextCluster, address, 1 << bl );
	RETF(ret);

   #if FAT12_SUPPORT
	if( pFS->FATType == FAT12 ) {
		if( cluster & 0x0001 )
			*pNextCluster >>= 4;
		else
			*pNextCluster &= 0x0FFF;

		if( *pNextCluster > FAT_BAD12 )
			return false;
	}
   #endif

   #if FAT32_SUPPORT
   	if( pFS->FATType == FAT32 ) {
   		if( *pNextCluster > FAT_BAD32 )
	   		return false;
   	} else
   #endif
	if( *pNextCluster > FAT_BAD16 )
		return false;
	
	return true;
}
						  
/**
 * This function returns the FAT type 12/16/32. That a valid FAT table
 * was found and does no additional validation (although this would be a
 * good place, if needed).
 */
FAT_TYPE_T _Fat_GetType(const fat_state_t* pFS, struct fat* pFAT) {
	char*	name = pFAT->vbr.fat16.fat_name;

	#if FAT_STRICT_TYPE_CHECK
		uint32_t		DataSec;
		fat_ca_t	ClusterCount;
		//
		// Calculate FAT type according to Microsoft white paper
		//
		DataSec = pFS->TotSec;
		DataSec -= pFS->RsvdSecCnt
		           + (pFS->NumFATs * pFS->FATSz)
		           + pFS->RootDirSz;
		
		// number of clusters in the data region
		ClusterCount = DataSec >> pFS->SecPerClus_bit;
		if( ClusterCount < 4085 ) {
			// not all devices format coforming to specification
			if( name[3] == '6' )
				return FAT16;	// ill-sized FAT16
			return FAT12;
		} else if( ClusterCount < 65525 )
			return FAT16;
		else
			return FAT32;
	#else
		// usually works, but is not reliable
		if( name[3] == '1' ) {
			if( name[4] == '6' )
				return FAT16;
			else if( name[4] == '2' )
				return FAT12;
		}
		return FAT32;
	#endif
}


