/*
Copyright 2006, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2006
*/

#include "ScatterWeb.MD2.h"
#include "ScatterWeb.MD2.internals.h"
#include <string.h>

void MD2_memcpy(uint8_t* output, uint8_t* input, uint16_t len) {
	memcpy(output, input, len);
}

void MD2_memset(uint8_t* output, uint16_t value, uint16_t len) {
	memset(output, value, len);
}

void MD2_init(MD2_CTX *context) {
	MD2_memset (context->state, 0, sizeof(context->state));
	MD2_memset (context->checksum, 0, sizeof(context->checksum));
	context->count = 0;
}

void MD2_transform(uint8_t *state, uint8_t *checksum, const uint8_t *block) {
	uint16_t i, j, t;
	uint8_t x[48];
	uint16_t *pxa16 = (uint16_t*) &x[32];
	uint16_t *ps16 = (uint16_t*) state;
	uint16_t *pb16 = (uint16_t*) block;
	// Form encryption block from state, block, state ^ block.
	MD2_memcpy(x, state, 16);
	MD2_memcpy(x+16, (uint8_t*) block, 16);
	for (i = 0; i < 8; i++) {
		*pxa16 = *ps16 ^ *pb16;
		++pxa16;
		++ps16;
		++pb16;
	}
	// Encrypt block (18 rounds)
	t = 0;
	for (i = 0; i < 18; i++) {
		for (j = 0; j < 48; j++) {
		t = x[j] ^= PI_SUBST[t];
		}
		t = (t + i) & 0xff;
	}
	// Save new state
	MD2_memcpy (state, x, 16);
	// Update checksum
	t = checksum[15];
	for (i = 0; i < 16; i++) {
		t = checksum[i] ^= PI_SUBST[block[i] ^ t];
	}
}

void MD2_update(MD2_CTX *context, const uint8_t *input, uint16_t inputLen) {
	uint16_t i, index, partLen;
	// Update number of bytes mod 16
	index = context->count;
	context->count = (index + inputLen) & 0xf;
	partLen = 16 - index;
	// Transform as many times as possible.
	if (inputLen >= partLen) {
		MD2_memcpy(&context->buffer[index], (uint8_t*) input, partLen);
		MD2_transform (context->state, context->checksum, context->buffer);
		for (i = partLen; i + 15 < inputLen; i += 16) {
		MD2_transform (context->state, context->checksum, &input[i]);
		}
		index = 0;
	}
	else {
		i = 0;
	}
	// Buffer remaining input
	MD2_memcpy(&context->buffer[index], (uint8_t*) &input[i], inputLen-i);
}

void MD2_final(uint8_t *digest, MD2_CTX *context) {
	uint16_t index, padLen;
	uint8_t pad[16];
	// Pad out to multiple of 16.
	index = context->count;
	padLen = 16 - index;
	MD2_memset(pad, padLen, padLen);
	MD2_update (context, pad, padLen);
	// Extend with checksum
	MD2_update (context, context->checksum, 16);
	// Store state in digest
	MD2_memcpy (digest, context->state, 16);
}

uint8_t* MD2_get(uint8_t *input, uint16_t length, uint8_t *output) {
	MD2_CTX context;
	MD2_init(&context);
	MD2_update(&context, input, length);
	MD2_final(output, &context);
	return output;
}
